@@ -41,14 +41,16 @@ struct _MMKernelDeviceClass {
     const gchar * (* get_driver)      (MMKernelDevice *self);
     const gchar * (* get_sysfs_path)  (MMKernelDevice *self);
 
-    gint          (* get_interface_class)      (MMKernelDevice *self);
-    gint          (* get_interface_subclass)   (MMKernelDevice *self);
-    gint          (* get_interface_protocol)   (MMKernelDevice *self);
-    const gchar * (* get_interface_sysfs_path) (MMKernelDevice *self);
+    gint          (* get_interface_class)       (MMKernelDevice *self);
+    gint          (* get_interface_subclass)    (MMKernelDevice *self);
+    gint          (* get_interface_protocol)    (MMKernelDevice *self);
+    const gchar * (* get_interface_sysfs_path)  (MMKernelDevice *self);
+    const gchar * (* get_interface_description) (MMKernelDevice *self);
 
     const gchar * (* get_physdev_uid) (MMKernelDevice *self);
     guint16       (* get_physdev_vid) (MMKernelDevice *self);
     guint16       (* get_physdev_pid) (MMKernelDevice *self);
+    guint16       (* get_physdev_revision)     (MMKernelDevice *self);
     const gchar * (* get_physdev_sysfs_path)   (MMKernelDevice *self);
     const gchar * (* get_physdev_subsystem)    (MMKernelDevice *self);
     const gchar * (* get_physdev_manufacturer) (MMKernelDevice *self);
@@ -56,34 +58,32 @@ struct _MMKernelDeviceClass {
 
     gboolean      (* cmp) (MMKernelDevice *a, MMKernelDevice *b);
 
-    gboolean      (* has_property)            (MMKernelDevice *self, const gchar *property);
-    const gchar * (* get_property)            (MMKernelDevice *self, const gchar *property);
-    gboolean      (* get_property_as_boolean) (MMKernelDevice *self, const gchar *property);
-    gint          (* get_property_as_int)     (MMKernelDevice *self, const gchar *property);
-    guint         (* get_property_as_int_hex) (MMKernelDevice *self, const gchar *property);
-
-    gboolean      (* has_global_property)            (MMKernelDevice *self, const gchar *property);
-    const gchar * (* get_global_property)            (MMKernelDevice *self, const gchar *property);
-    gboolean      (* get_global_property_as_boolean) (MMKernelDevice *self, const gchar *property);
-    gint          (* get_global_property_as_int)     (MMKernelDevice *self, const gchar *property);
-    guint         (* get_global_property_as_int_hex) (MMKernelDevice *self, const gchar *property);
+    gboolean      (* has_property)        (MMKernelDevice *self, const gchar *property);
+    const gchar * (* get_property)        (MMKernelDevice *self, const gchar *property);
+    gboolean      (* has_global_property) (MMKernelDevice *self, const gchar *property);
+    const gchar * (* get_global_property) (MMKernelDevice *self, const gchar *property);
+    gboolean      (* has_attribute)       (MMKernelDevice *self, const gchar *attribute);
+    const gchar * (* get_attribute)       (MMKernelDevice *self, const gchar *attribute);
 };
 
 GType mm_kernel_device_get_type (void);
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (MMKernelDevice, g_object_unref)
 
 const gchar *mm_kernel_device_get_subsystem   (MMKernelDevice *self);
 const gchar *mm_kernel_device_get_name        (MMKernelDevice *self);
 const gchar *mm_kernel_device_get_driver      (MMKernelDevice *self);
 const gchar *mm_kernel_device_get_sysfs_path  (MMKernelDevice *self);
 
-gint         mm_kernel_device_get_interface_class      (MMKernelDevice *self);
-gint         mm_kernel_device_get_interface_subclass   (MMKernelDevice *self);
-gint         mm_kernel_device_get_interface_protocol   (MMKernelDevice *self);
-const gchar *mm_kernel_device_get_interface_sysfs_path (MMKernelDevice *self);
+gint         mm_kernel_device_get_interface_class       (MMKernelDevice *self);
+gint         mm_kernel_device_get_interface_subclass    (MMKernelDevice *self);
+gint         mm_kernel_device_get_interface_protocol    (MMKernelDevice *self);
+const gchar *mm_kernel_device_get_interface_sysfs_path  (MMKernelDevice *self);
+const gchar *mm_kernel_device_get_interface_description (MMKernelDevice *self);
 
 const gchar *mm_kernel_device_get_physdev_uid          (MMKernelDevice *self);
 guint16      mm_kernel_device_get_physdev_vid          (MMKernelDevice *self);
 guint16      mm_kernel_device_get_physdev_pid          (MMKernelDevice *self);
+guint16      mm_kernel_device_get_physdev_revision     (MMKernelDevice *self);
 const gchar *mm_kernel_device_get_physdev_sysfs_path   (MMKernelDevice *self);
 const gchar *mm_kernel_device_get_physdev_subsystem    (MMKernelDevice *self);
 const gchar *mm_kernel_device_get_physdev_manufacturer (MMKernelDevice *self);
@@ -105,4 +105,11 @@ gboolean     mm_kernel_device_get_global_property_as_boolean (MMKernelDevice *se
 gint         mm_kernel_device_get_global_property_as_int     (MMKernelDevice *self, const gchar *property);
 guint        mm_kernel_device_get_global_property_as_int_hex (MMKernelDevice *self, const gchar *property);
 
+/* Attributes in sysfs */
+gboolean     mm_kernel_device_has_attribute            (MMKernelDevice *self, const gchar *attribute);
+const gchar *mm_kernel_device_get_attribute            (MMKernelDevice *self, const gchar *attribute);
+gboolean     mm_kernel_device_get_attribute_as_boolean (MMKernelDevice *self, const gchar *attribute);
+gint         mm_kernel_device_get_attribute_as_int     (MMKernelDevice *self, const gchar *attribute);
+guint        mm_kernel_device_get_attribute_as_int_hex (MMKernelDevice *self, const gchar *attribute);
+
 #endif /* MM_KERNEL_DEVICE_H */
