@@ -237,6 +237,7 @@ struct _MMIfaceModem3gpp {
 };
 
 GType mm_iface_modem_3gpp_get_type (void);
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (MMIfaceModem3gpp, g_object_unref)
 
 /* Initialize Modem 3GPP interface (async) */
 void     mm_iface_modem_3gpp_initialize        (MMIfaceModem3gpp *self,
@@ -291,6 +292,7 @@ void mm_iface_modem_3gpp_update_pco_list            (MMIfaceModem3gpp *self,
                                                      const GList *pco_list);
 void mm_iface_modem_3gpp_update_initial_eps_bearer  (MMIfaceModem3gpp *self,
                                                      MMBearerProperties *properties);
+void mm_iface_modem_3gpp_reload_initial_eps_bearer  (MMIfaceModem3gpp *self);
 
 /* Run all registration checks */
 void mm_iface_modem_3gpp_run_registration_checks (MMIfaceModem3gpp *self,
