@@ -19,7 +19,7 @@
 
 #include "mm-iface-modem.h"
 #include "mm-iface-modem-time.h"
-#include "mm-log.h"
+#include "mm-log-object.h"
 
 #define SUPPORT_CHECKED_TAG          "time-support-checked-tag"
 #define SUPPORTED_TAG                "time-supported-tag"
@@ -55,42 +55,34 @@ handle_get_network_time_context_free (HandleGetNetworkTimeContext *ctx)
 }
 
 static void
-load_network_time_ready (MMIfaceModemTime *self,
-                         GAsyncResult *res,
+load_network_time_ready (MMIfaceModemTime            *self,
+                         GAsyncResult                *res,
                          HandleGetNetworkTimeContext *ctx)
 {
     gchar *time_str;
     GError *error = NULL;
 
-    time_str = MM_IFACE_MODEM_TIME_GET_INTERFACE (self)->load_network_time_finish (self,
-                                                                                   res,
-                                                                                   &error);
+    time_str = MM_IFACE_MODEM_TIME_GET_INTERFACE (self)->load_network_time_finish (self, res, &error);
     if (error)
         g_dbus_method_invocation_take_error (ctx->invocation, error);
     else
-        mm_gdbus_modem_time_complete_get_network_time (ctx->skeleton,
-                                                       ctx->invocation,
-                                                       time_str);
+        mm_gdbus_modem_time_complete_get_network_time (ctx->skeleton, ctx->invocation, time_str);
     g_free (time_str);
     handle_get_network_time_context_free (ctx);
 }
 
-static gboolean
-handle_get_network_time (MmGdbusModemTime *skeleton,
-                         GDBusMethodInvocation *invocation,
-                         MMIfaceModemTime *self)
+static void
+handle_get_network_time_auth_ready (MMBaseModem                 *self,
+                                    GAsyncResult                *res,
+                                    HandleGetNetworkTimeContext *ctx)
 {
-    HandleGetNetworkTimeContext *ctx;
-    MMModemState state;
+    MMModemState  state;
+    GError       *error = NULL;
 
-    if (!MM_IFACE_MODEM_TIME_GET_INTERFACE (self)->load_network_time ||
-        !MM_IFACE_MODEM_TIME_GET_INTERFACE (self)->load_network_time_finish) {
-        g_dbus_method_invocation_return_error (invocation,
-                                               MM_CORE_ERROR,
-                                               MM_CORE_ERROR_UNSUPPORTED,
-                                               "Cannot load network time: "
-                                               "operation not supported");
-        return TRUE;
+    if (!mm_base_modem_authorize_finish (self, res, &error)) {
+        g_dbus_method_invocation_take_error (ctx->invocation, error);
+        handle_get_network_time_context_free (ctx);
+        return;
     }
 
     state = MM_MODEM_STATE_UNKNOWN;
@@ -99,23 +91,49 @@ handle_get_network_time (MmGdbusModemTime *skeleton,
                   NULL);
     /* If we're not yet registered, we cannot get the network time */
     if (state < MM_MODEM_STATE_REGISTERED) {
-        g_dbus_method_invocation_return_error (invocation,
+        g_dbus_method_invocation_return_error (ctx->invocation,
                                                MM_CORE_ERROR,
                                                MM_CORE_ERROR_WRONG_STATE,
                                                "Cannot load network time: "
                                                "not registered yet");
-        return TRUE;
+        handle_get_network_time_context_free (ctx);
+        return;
     }
 
+    if (!MM_IFACE_MODEM_TIME_GET_INTERFACE (self)->load_network_time ||
+        !MM_IFACE_MODEM_TIME_GET_INTERFACE (self)->load_network_time_finish) {
+        g_dbus_method_invocation_return_error (ctx->invocation,
+                                               MM_CORE_ERROR,
+                                               MM_CORE_ERROR_UNSUPPORTED,
+                                               "Cannot load network time: "
+                                               "operation not supported");
+        handle_get_network_time_context_free (ctx);
+        return;
+    }
+
+    MM_IFACE_MODEM_TIME_GET_INTERFACE (self)->load_network_time (
+        ctx->self,
+        (GAsyncReadyCallback)load_network_time_ready,
+        ctx);
+}
+
+static gboolean
+handle_get_network_time (MmGdbusModemTime      *skeleton,
+                         GDBusMethodInvocation *invocation,
+                         MMIfaceModemTime      *self)
+{
+    HandleGetNetworkTimeContext *ctx;
+
     ctx = g_new (HandleGetNetworkTimeContext, 1);
     ctx->invocation = g_object_ref (invocation);
     ctx->skeleton = g_object_ref (skeleton);
     ctx->self = g_object_ref (self);
 
-    MM_IFACE_MODEM_TIME_GET_INTERFACE (self)->load_network_time (
-        self,
-        (GAsyncReadyCallback)load_network_time_ready,
-        ctx);
+    mm_base_modem_authorize (MM_BASE_MODEM (self),
+                             invocation,
+                             MM_AUTHORIZATION_TIME,
+                             (GAsyncReadyCallback)handle_get_network_time_auth_ready,
+                             ctx);
     return TRUE;
 }
 
@@ -176,7 +194,7 @@ static void
 load_network_timezone_ready (MMIfaceModemTime *self,
                              GAsyncResult *res)
 {
-    GError *error = NULL;
+    g_autoptr(GError)  error = NULL;
     MMNetworkTimezone *tz;
 
     /* Finish the async operation */
@@ -184,8 +202,7 @@ load_network_timezone_ready (MMIfaceModemTime *self,
     if (!tz) {
         NetworkTimezoneContext *ctx;
 
-        mm_dbg ("Couldn't load network timezone: %s", error->message);
-        g_error_free (error);
+        mm_obj_dbg (self, "couldn't load network timezone: %s", error->message);
 
         /* Note: may be NULL if the polling has been removed while processing the async operation */
         ctx = (NetworkTimezoneContext *) g_object_get_qdata (G_OBJECT (self), network_timezone_context_quark);
@@ -198,7 +215,7 @@ load_network_timezone_ready (MMIfaceModemTime *self,
         /* If no more retries, we don't do anything else */
         if (ctx->network_timezone_poll_retries == 0 ||
             !g_error_matches (error, MM_CORE_ERROR, MM_CORE_ERROR_RETRY)) {
-            mm_warn ("Couldn't load network timezone from the current network");
+            mm_obj_warn (self, "couldn't load network timezone from the current network");
             return;
         }
 
@@ -237,7 +254,7 @@ start_network_timezone_poll (MMIfaceModemTime *self)
 
     ctx = (NetworkTimezoneContext *) g_object_get_qdata (G_OBJECT (self), network_timezone_context_quark);
 
-    mm_dbg ("Network timezone polling started");
+    mm_obj_dbg (self, "network timezone polling started");
     ctx->network_timezone_poll_retries = NETWORK_TIMEZONE_POLL_RETRIES;
     ctx->network_timezone_poll_id = g_timeout_add_seconds (NETWORK_TIMEZONE_POLL_INTERVAL_SEC, (GSourceFunc)network_timezone_poll_cb, self);
 }
@@ -250,7 +267,7 @@ stop_network_timezone_poll (MMIfaceModemTime *self)
     ctx = (NetworkTimezoneContext *) g_object_get_qdata (G_OBJECT (self), network_timezone_context_quark);
 
     if (ctx->network_timezone_poll_id) {
-        mm_dbg ("Network timezone polling stopped");
+        mm_obj_dbg (self, "network timezone polling stopped");
         g_source_remove (ctx->network_timezone_poll_id);
         ctx->network_timezone_poll_id = 0;
     }
@@ -280,6 +297,22 @@ network_timezone_state_changed (MMIfaceModemTime *self)
     }
 }
 
+static void
+stop_network_timezone (MMIfaceModemTime *self)
+{
+    NetworkTimezoneContext *ctx;
+
+    ctx = (NetworkTimezoneContext *) g_object_get_qdata (G_OBJECT (self), network_timezone_context_quark);
+    if (ctx) {
+        /* Remove signal connection and then trigger context free */
+        if (ctx->state_changed_id) {
+            g_signal_handler_disconnect (self, ctx->state_changed_id);
+            ctx->state_changed_id = 0;
+        }
+        g_object_set_qdata (G_OBJECT (self), network_timezone_context_quark, NULL);
+    }
+}
+
 static void
 start_network_timezone (MMIfaceModemTime *self)
 {
@@ -288,13 +321,16 @@ start_network_timezone (MMIfaceModemTime *self)
     /* If loading network timezone not supported, just finish here */
     if (!MM_IFACE_MODEM_TIME_GET_INTERFACE (self)->load_network_timezone ||
         !MM_IFACE_MODEM_TIME_GET_INTERFACE (self)->load_network_timezone_finish) {
-        mm_dbg ("Loading network timezone is not supported");
+        mm_obj_dbg (self, "loading network timezone is not supported");
         return;
     }
 
     if (G_UNLIKELY (!network_timezone_context_quark))
         network_timezone_context_quark = (g_quark_from_static_string (NETWORK_TIMEZONE_CONTEXT_TAG));
 
+    /* Cleanup context properly if it already exists, including the signal handler */
+    stop_network_timezone (self);
+
     ctx = g_new0 (NetworkTimezoneContext, 1);
     g_object_set_qdata_full (G_OBJECT (self),
                              network_timezone_context_quark,
@@ -315,22 +351,6 @@ start_network_timezone (MMIfaceModemTime *self)
         start_network_timezone_poll (self);
 }
 
-static void
-stop_network_timezone (MMIfaceModemTime *self)
-{
-    NetworkTimezoneContext *ctx;
-
-    ctx = (NetworkTimezoneContext *) g_object_get_qdata (G_OBJECT (self), network_timezone_context_quark);
-    if (ctx) {
-        /* Remove signal connection and then trigger context free */
-        if (ctx->state_changed_id) {
-            g_signal_handler_disconnect (self, ctx->state_changed_id);
-            ctx->state_changed_id = 0;
-        }
-        g_object_set_qdata (G_OBJECT (self), network_timezone_context_quark, NULL);
-    }
-}
-
 /*****************************************************************************/
 
 void
@@ -351,6 +371,27 @@ mm_iface_modem_time_update_network_time (MMIfaceModemTime *self,
     g_object_unref (skeleton);
 }
 
+void
+mm_iface_modem_time_update_network_timezone (MMIfaceModemTime  *self,
+                                             MMNetworkTimezone *tz)
+{
+    MmGdbusModemTime *skeleton;
+    GVariant         *dictionary;
+
+    g_object_get (self,
+                  MM_IFACE_MODEM_TIME_DBUS_SKELETON, &skeleton,
+                  NULL);
+    if (!skeleton)
+        return;
+
+    dictionary = mm_network_timezone_get_dictionary (tz);
+    mm_gdbus_modem_time_set_network_timezone (skeleton, dictionary);
+    if (dictionary)
+        g_variant_unref (dictionary);
+
+    g_object_unref (skeleton);
+}
+
 /*****************************************************************************/
 
 typedef struct _DisablingContext DisablingContext;
@@ -438,14 +479,14 @@ interface_disabling_step (GTask *task)
 
     switch (ctx->step) {
     case DISABLING_STEP_FIRST:
-        /* Fall down to next step */
         ctx->step++;
+        /* fall through */
 
     case DISABLING_STEP_CANCEL_NETWORK_TIMEZONE_UPDATE:
         /* Stop and cleanup context */
         stop_network_timezone (self);
-        /* Fall down to next step */
         ctx->step++;
+        /* fall through */
 
     case DISABLING_STEP_DISABLE_UNSOLICITED_EVENTS:
         /* Allow cleaning up unsolicited events */
@@ -457,8 +498,8 @@ interface_disabling_step (GTask *task)
                 task);
             return;
         }
-        /* Fall down to next step */
         ctx->step++;
+        /* fall through */
 
     case DISABLING_STEP_CLEANUP_UNSOLICITED_EVENTS:
         /* Allow cleaning up unsolicited events */
@@ -470,14 +511,17 @@ interface_disabling_step (GTask *task)
                 task);
             return;
         }
-        /* Fall down to next step */
         ctx->step++;
+        /* fall through */
 
     case DISABLING_STEP_LAST:
         /* We are done without errors! */
         g_task_return_boolean (task, TRUE);
         g_object_unref (task);
         return;
+
+    default:
+        break;
     }
 
     g_assert_not_reached ();
@@ -577,7 +621,7 @@ enable_unsolicited_events_ready (MMIfaceModemTime *self,
 
     /* Not critical! */
     if (!MM_IFACE_MODEM_TIME_GET_INTERFACE (self)->enable_unsolicited_events_finish (self, res, &error)) {
-        mm_dbg ("Couldn't enable unsolicited events: '%s'", error->message);
+        mm_obj_dbg (self, "couldn't enable unsolicited events: %s", error->message);
         g_error_free (error);
     }
 
@@ -604,14 +648,14 @@ interface_enabling_step (GTask *task)
 
     switch (ctx->step) {
     case ENABLING_STEP_FIRST:
-        /* Fall down to next step */
         ctx->step++;
+        /* fall through */
 
     case ENABLING_STEP_SETUP_NETWORK_TIMEZONE_RETRIEVAL:
         /* We start it and schedule it to run asynchronously */
         start_network_timezone (self);
-        /* Fall down to next step */
         ctx->step++;
+        /* fall through */
 
     case ENABLING_STEP_SETUP_UNSOLICITED_EVENTS:
         /* Allow setting up unsolicited events */
@@ -623,8 +667,8 @@ interface_enabling_step (GTask *task)
                 task);
             return;
         }
-        /* Fall down to next step */
         ctx->step++;
+        /* fall through */
 
     case ENABLING_STEP_ENABLE_UNSOLICITED_EVENTS:
         /* Allow setting up unsolicited events */
@@ -636,14 +680,17 @@ interface_enabling_step (GTask *task)
                 task);
             return;
         }
-        /* Fall down to next step */
         ctx->step++;
+        /* fall through */
 
     case ENABLING_STEP_LAST:
         /* We are done without errors! */
         g_task_return_boolean (task, TRUE);
         g_object_unref (task);
         return;
+
+    default:
+        break;
     }
 
     g_assert_not_reached ();
@@ -716,7 +763,7 @@ check_support_ready (MMIfaceModemTime *self,
                                                                          &error)) {
         if (error) {
             /* This error shouldn't be treated as critical */
-            mm_dbg ("Time support check failed: '%s'", error->message);
+            mm_obj_dbg (self, "time support check failed: %s", error->message);
             g_error_free (error);
         }
     } else {
@@ -757,8 +804,8 @@ interface_initialization_step (GTask *task)
             supported_quark = (g_quark_from_static_string (
                                    SUPPORTED_TAG));
 
-        /* Fall down to next step */
         ctx->step++;
+        /* fall through */
 
     case INITIALIZATION_STEP_CHECK_SUPPORT:
         if (!GPOINTER_TO_UINT (g_object_get_qdata (G_OBJECT (self),
@@ -784,8 +831,8 @@ interface_initialization_step (GTask *task)
             /* If there is no implementation to check support, assume we DON'T
              * support it. */
         }
-        /* Fall down to next step */
         ctx->step++;
+        /* fall through */
 
     case INITIALIZATION_STEP_FAIL_IF_UNSUPPORTED:
         if (!GPOINTER_TO_UINT (g_object_get_qdata (G_OBJECT (self),
@@ -797,8 +844,8 @@ interface_initialization_step (GTask *task)
             g_object_unref (task);
             return;
         }
-        /* Fall down to next step */
         ctx->step++;
+        /* fall through */
 
     case INITIALIZATION_STEP_LAST:
         /* We are done without errors! */
@@ -816,6 +863,9 @@ interface_initialization_step (GTask *task)
         g_task_return_boolean (task, TRUE);
         g_object_unref (task);
         return;
+
+    default:
+        break;
     }
 
     g_assert_not_reached ();
